# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.xla.experimental namespace.
"""

import sys as _sys

from tensorflow.python.compiler.xla.jit import experimental_jit_scope as jit_scope
from tensorflow.python.compiler.xla.xla import compile