# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/core_platform_payloads.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow/core/protobuf/core_platform_payloads.proto',
  package='tensorflow.core.platform',
  syntax='proto3',
  serialized_options=_b('ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'),
  serialized_pb=_b('\n5tensorflow/core/protobuf/core_platform_payloads.proto\x12\x18tensorflow.core.platform\"\xec\x01\n\x10\x45rrorSourceProto\x12L\n\x0c\x65rror_source\x18\x01 \x01(\x0e\x32\x36.tensorflow.core.platform.ErrorSourceProto.ErrorSource\"\x89\x01\n\x0b\x45rrorSource\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x12\n\x0eTPU_COMPILE_OP\x10\x01\x12\x11\n\rTF_XLA_BRIDGE\x10\x02\x12\x17\n\x13MLIR_BRIDGE_PHASE_1\x10\x03\x12\x17\n\x13MLIR_BRIDGE_PHASE_2\x10\x04\x12\x14\n\x10\x45\x41GER_REMOTE_MGR\x10\x05\x42ZZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')
)



_ERRORSOURCEPROTO_ERRORSOURCE = _descriptor.EnumDescriptor(
  name='ErrorSource',
  full_name='tensorflow.core.platform.ErrorSourceProto.ErrorSource',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TPU_COMPILE_OP', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='TF_XLA_BRIDGE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MLIR_BRIDGE_PHASE_1', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MLIR_BRIDGE_PHASE_2', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EAGER_REMOTE_MGR', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=183,
  serialized_end=320,
)
_sym_db.RegisterEnumDescriptor(_ERRORSOURCEPROTO_ERRORSOURCE)


_ERRORSOURCEPROTO = _descriptor.Descriptor(
  name='ErrorSourceProto',
  full_name='tensorflow.core.platform.ErrorSourceProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_source', full_name='tensorflow.core.platform.ErrorSourceProto.error_source', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ERRORSOURCEPROTO_ERRORSOURCE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=84,
  serialized_end=320,
)

_ERRORSOURCEPROTO.fields_by_name['error_source'].enum_type = _ERRORSOURCEPROTO_ERRORSOURCE
_ERRORSOURCEPROTO_ERRORSOURCE.containing_type = _ERRORSOURCEPROTO
DESCRIPTOR.message_types_by_name['ErrorSourceProto'] = _ERRORSOURCEPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ErrorSourceProto = _reflection.GeneratedProtocolMessageType('ErrorSourceProto', (_message.Message,), {
  'DESCRIPTOR' : _ERRORSOURCEPROTO,
  '__module__' : 'tensorflow.core.protobuf.core_platform_payloads_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.core.platform.ErrorSourceProto)
  })
_sym_db.RegisterMessage(ErrorSourceProto)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
