# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/function/trace_type/serialization.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow/core/function/trace_type/serialization.proto',
  package='tensorflow.core.function.trace_type.serialization',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n7tensorflow/core/function/trace_type/serialization.proto\x12\x31tensorflow.core.function.trace_type.serialization\x1a\x19google/protobuf/any.proto\"C\n\x13SerializedTraceType\x12,\n\x0erepresentation\x18\x01 \x01(\x0b\x32\x14.google.protobuf.Any')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_SERIALIZEDTRACETYPE = _descriptor.Descriptor(
  name='SerializedTraceType',
  full_name='tensorflow.core.function.trace_type.serialization.SerializedTraceType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='representation', full_name='tensorflow.core.function.trace_type.serialization.SerializedTraceType.representation', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=204,
)

_SERIALIZEDTRACETYPE.fields_by_name['representation'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['SerializedTraceType'] = _SERIALIZEDTRACETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SerializedTraceType = _reflection.GeneratedProtocolMessageType('SerializedTraceType', (_message.Message,), {
  'DESCRIPTOR' : _SERIALIZEDTRACETYPE,
  '__module__' : 'tensorflow.core.function.trace_type.serialization_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.core.function.trace_type.serialization.SerializedTraceType)
  })
_sym_db.RegisterMessage(SerializedTraceType)


# @@protoc_insertion_point(module_scope)
